﻿#NoEnv  ; Recommended for performance and compatibility with future AutoHotkey releases.
#Warn  ; Recommended for catching common errors.
#SingleInstance
#NoTrayIcon

;@Ahk2Exe-SetName Explorer Restart
;@Ahk2Exe-SetDescription Restarts File Explorer
;@Ahk2Exe-SetVersion 1.0
;@Ahk2Exe-SetCopyright ©2021 Daniel Beardsmore
;@Ahk2Exe-SetOrigFilename Explorer Restart.exe
;@Ahk2Exe-SetMainIcon Explorer Restart.ico

global PROGRESS_INIT					:= 0
global PROGRESS_CACHED				:= 15
global PROGRESS_WAIT_WIN_CLOSE	:= 35
global PROGRESS_CLOSE_EXPLORER	:= 45
global PROGRESS_START_EXPLORER	:= 65
global PROGRESS_REOPEN_WINDOWS	:= 85
global PROGRESS_DONE					:= 100

global READABILITY_DELAY_MS		:= 25

Progress, M T FS8 W640, Caching open windows, Restarting Explorer, Explorer Restart

paths := []
windowCount := 0
progressSlice := 0

; Preserve all existing windows
appObj := ComObjCreate("Shell.Application")
for window in appObj.Windows
{
	windowCount++
}

i := 0
progressSlice := Floor((PROGRESS_CACHED - PROGRESS_INIT) / windowCount)

for window in appObj.Windows
{
	paths.push(window.Document.Folder.Self.Path)
	i++
	Progress, % PROGRESS_INIT + (i * progressSlice), % "Saved window " . window.Document.Folder.Self.Path
	Sleep, % READABILITY_DELAY_MS
}

; Close all Explorer windows
; From https://autohotkey.com/board/topic/115359-closing-a-simple-explorer-window-after-operations/
Progress, % PROGRESS_CACHED, Closing open windows …

i := 0
progressSlice := Floor((PROGRESS_WAIT_WIN_CLOSE - PROGRESS_CACHED) / windowCount)


While (WinExist("ahk_Class CabinetWClass")) {
	WinClose, ahk_class CabinetWClass
	i++
	Progress, % PROGRESS_CACHED + (i * progressSlice)
	Sleep, % READABILITY_DELAY_MS
}

; Allow time for windows to close before signalling process exit
; Probably unnecessary but it can’t do any harm
Progress, % PROGRESS_WAIT_WIN_CLOSE, Allowing time for Explorer to respond …
Sleep, 750

; Quit Explorer; this just closes the taskbar, but if all Explorer windows are already closed (above), Explorer then exits
; From https://autohotkey.com/board/topic/93906-restart-explorer-the-official-way/
Progress, % PROGRESS_CLOSE_EXPLORER, Closing Explorer …
WinGet, h, ID, ahk_class Progman
PostMessage, 0x12, 0, 0, , ahk_id %h%	;wm_quit

; Allow time for the outgoing Explorer process to close before launching it, and terminate it if it never exits.
; If it does not close, we get into a crash loop that is hard to recover from.
; We allow another sleep here to ensure that it fully terminates — it seems that possibly the crash loop is caused
; by the shell trying to signal the closing process and causing mayhem in the process
Process, WaitClose, explorer.exe, 7
Process, Close, explorer.exe
Sleep, 750

; Restart Explorer
Progress, % PROGRESS_START_EXPLORER, Starting Explorer …
Run, explorer.exe

; Allow some time for it to launch before opening windows
Sleep, 1500

; Re-open all previously-opened windows
Progress, % PROGRESS_REOPEN_WINDOWS, Re-opening windows …
progressSlice := Floor((PROGRESS_DONE - PROGRESS_REOPEN_WINDOWS) / windowCount)
for i, path in paths {
	Progress, % PROGRESS_REOPEN_WINDOWS + (progressSlice * i), % "Opening window for " . path
	Run, "%path%"
	Sleep, % READABILITY_DELAY_MS
}

Progress, % PROGRESS_DONE, % "✓"
Sleep, 750